# Default scales -------------------------------------------------------------

#' @export
#' @rdname scale_hue
scale_colour_discrete <- scale_colour_hue

#' @export
#' @rdname scale_gradient
scale_colour_continuous <- scale_colour_gradient

#' @export
#' @rdname scale_hue
scale_fill_discrete <- scale_fill_hue

#' @export
#' @rdname scale_gradient
scale_fill_continuous <- scale_fill_gradient

# British to American spellings ----------------------------------------------

#' @export
#' @rdname scale_brewer
scale_color_brewer <- scale_colour_brewer

#' @export
#' @rdname scale_gradient
scale_color_continuous <- scale_colour_gradient

#' @export
#' @rdname scale_hue
scale_color_discrete <- scale_colour_hue

#' @export
#' @rdname scale_gradient
scale_color_gradient <- scale_colour_gradient

#' @export
#' @rdname scale_gradient2
scale_color_gradient2 <- scale_colour_gradient2

#' @export
#' @rdname scale_gradientn
scale_color_gradientn <- scale_colour_gradientn

#' @export
#' @rdname scale_grey
scale_color_grey <- scale_colour_grey

#' @export
#' @rdname scale_hue
scale_color_hue <- scale_colour_hue

#' @export
#' @rdname scale_identity
scale_color_identity <- scale_colour_identity

#' @export
#' @rdname scale_manual
scale_color_manual <- scale_colour_manual
